﻿using System;
using MonoDevelop.Components.Commands;
using MonoDevelop.Ide;
using System.IO;
using System.Diagnostics;
using MonoDevelop.Core;
using System.Collections.Generic;

namespace ForGetMonoDevelopAddin
{

	class BuildDetector : CommandHandler
	{
		public BuildDetector ()
		{
		}

		protected override void Run ()
		{
			MonoDevelop.Ide.IdeApp.ProjectOperations.StartBuild += StartBuildHandler;;
		}

		void StartBuildHandler (object sender, MonoDevelop.Projects.BuildEventArgs args)
		{
			ProjectOperations projectOperations = sender as ProjectOperations;
			if (projectOperations == null) 
			{
				return;
			}
			var solutionDir = projectOperations.CurrentSelectedSolution.BaseDirectory.ToString();

			string depedenciesFile = Path.Combine(solutionDir, "dependencies.config");
			if(!File.Exists(depedenciesFile))
			{
				return; //this solution is not using ForGet
			}
			_messages.Clear ();
			using (var monitor = MonoDevelop.Ide.IdeApp.Workbench.ProgressMonitors.GetBuildProgressMonitor()) 
			using (var process = new System.Diagnostics.Process())
			{
				monitor.BeginTask ("Restoring ForGet Packages", 1);
				process.StartInfo = new ProcessStartInfo()
				{
					FileName = "ForGet",
					Arguments = "-r",
					WorkingDirectory = solutionDir,
					RedirectStandardError = true,
					RedirectStandardOutput = true,
					UseShellExecute = false,
				};

				process.OutputDataReceived += ProcessDataReceived;
				process.ErrorDataReceived += ProcessDataReceived;
				if (!process.Start())
				{
					WriteToBuildOutput(monitor, "ERROR: Failed to start ForGet");
					return;
				}
				process.BeginOutputReadLine();
				process.BeginErrorReadLine();
				process.WaitForExit();
				foreach (var message in _messages) 
				{
					WriteToBuildOutput(monitor, message);
				}
				if (process.ExitCode != 0)
				{
					WriteToBuildOutput(monitor, "ERROR: ForGet.exe failed");
					return;
				}
				monitor.EndTask ();
				return;
			}
		}

		List<string> _messages = new List<string>();

		void WriteToBuildOutput(IProgressMonitor monitor, string message)
		{
			monitor.Log.WriteLine (message);
		}

		void ProcessDataReceived(object sender, DataReceivedEventArgs e)
		{
			if (e.Data != null)
			{
				if (e.Data.Length != 0)
				{
					_messages.Add (string.Format("    {0}", e.Data));
				}
			}
		}
	}
}

